@@ -1,5 +1,5 @@
 /* NBD client library in userspace.
- * Copyright (C) 2020-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -64,7 +64,7 @@
 #endif
 
 #ifdef PAGE_CACHE_MAPPING
-DEFINE_VECTOR_TYPE (byte_vector, uint8_t)
+DEFINE_VECTOR_TYPE (byte_vector, uint8_t);
 #endif
 
 static struct rw_ops file_ops;
@@ -90,7 +90,7 @@ struct rw_file {
 #ifdef PAGE_CACHE_MAPPING
 static long page_size;
 
-static void page_size_init (void) __attribute__((constructor));
+static void page_size_init (void) __attribute__ ((constructor));
 static void
 page_size_init (void)
 {
@@ -334,7 +334,7 @@ file_close (struct rw *rw)
 static void
 file_truncate (struct rw *rw, int64_t size)
 {
-  struct rw_file *rwf = (struct rw_file *) rw;
+  struct rw_file *rwf = (struct rw_file *)rw;
 
   /* If the destination is an ordinary file then the original file
    * size doesn't matter.  Truncate it to the source size.  But
@@ -351,8 +351,8 @@ file_truncate (struct rw *rw, int64_t size)
   }
   rwf->rw.size = size;
 
-  /* We can assume the destination is zero. */
-  destination_is_zero = true;
+  /* We can assume the target is zero. */
+  target_is_zero = true;
 }
 
 static void
@@ -420,7 +420,7 @@ file_synch_read (struct rw *rw,
     if (r == 0)
       return n;
 
-    data += r;
+    data = (char *)data + r;
     offset += r;
     len -= r;
     n += r;
@@ -450,7 +450,7 @@ file_synch_write (struct rw *rw,
       perror (rw->name);
       exit (EXIT_FAILURE);
     }
-    data += r;
+    data = (char *)data + r;
     offset += r;
     len -= r;
   }
