@@ -7,9 +7,11 @@
  *
  * To run it:
  *
- *     nbdkit -r pattern size=1G -U /tmp/src.sock
- *     nbdkit memory size=1g -U /tmp/dst.sock
- *     ./copy-ev nbd+unix:///?socket=/tmp/src.sock nbd+unix:///?socket=/tmp/dst.sock
+ *     SRC=/tmp/src.sock
+ *     DST=/tmp/dst.sock
+ *     nbdkit -r pattern size=1G -U $SRC
+ *     nbdkit memory size=1g -U $DST
+ *     ./copy-libev nbd+unix:///?socket=$SRC nbd+unix:///?socket=$DST
  *
  * To debug it:
  *
@@ -121,12 +123,12 @@ static ev_timer progress;
 
 static inline void start_request_soon (struct request *r);
 static void start_request_cb (struct ev_loop *loop, ev_timer *w, int revents);
-static void start_request(struct request *r);
-static void start_read(struct request *r);
-static void start_write(struct request *r);
-static void start_zero(struct request *r);
-static int read_completed(void *user_data, int *error);
-static int request_completed(void *user_data, int *error);
+static void start_request (struct request *r);
+static void start_read (struct request *r);
+static void start_write (struct request *r);
+static void start_zero (struct request *r);
+static int read_completed (void *user_data, int *error);
+static int request_completed (void *user_data, int *error);
 
 /* Return true iff data is all zero bytes.
  *
@@ -134,7 +136,7 @@ static int request_completed(void *user_data, int *error);
  * https://rusty.ozlabs.org/?p=560
  */
 static bool
-is_zero (const unsigned char *data, size_t len)
+is_buf_zero (const unsigned char *data, size_t len)
 {
   const unsigned char *p = data;
   size_t i;
@@ -158,13 +160,13 @@ request_state (struct request *r)
 }
 
 static inline int
-get_fd(struct connection *c)
+get_fd (struct connection *c)
 {
     return nbd_aio_get_fd (c->nbd);
 }
 
 static inline int
-get_events(struct connection *c)
+get_events (struct connection *c)
 {
     unsigned dir = nbd_aio_get_direction (c->nbd);
 
@@ -388,7 +390,7 @@ start_request_cb (struct ev_loop *loop, ev_timer *w, int revents)
 
 /* Start async copy or zero request. */
 static void
-start_request(struct request *r)
+start_request (struct request *r)
 {
     /* Cancel the request if we are done. */
     if (offset == size)
@@ -430,7 +432,7 @@ start_request(struct request *r)
 }
 
 static void
-start_read(struct request *r)
+start_read (struct request *r)
 {
     int64_t cookie;
 
@@ -459,7 +461,7 @@ read_completed (void *user_data, int *error)
     DEBUG ("r%zu: read completed offset=%" PRIi64 " len=%zu",
            r->index, r->offset, r->length);
 
-    if (dst.can_zero && is_zero (r->data, r->length))
+    if (dst.can_zero && is_buf_zero (r->data, r->length))
         start_zero (r);
     else
         start_write (r);
@@ -468,7 +470,7 @@ read_completed (void *user_data, int *error)
 }
 
 static void
-start_write(struct request *r)
+start_write (struct request *r)
 {
     int64_t cookie;
 
@@ -487,7 +489,7 @@ start_write(struct request *r)
 }
 
 static void
-start_zero(struct request *r)
+start_zero (struct request *r)
 {
     int64_t cookie;
 
@@ -535,7 +537,7 @@ request_completed (void *user_data, int *error)
      * iteration, to avoid deadlock if we need to start a zero or write.
      */
     if (offset < size)
-        start_request_soon(r);
+        start_request_soon (r);
 
     return 1;
 }
@@ -590,7 +592,7 @@ finish_progress ()
 static inline void
 update_watcher (struct connection *c)
 {
-    int events = get_events(c);
+    int events = get_events (c);
 
     if (events != c->watcher.events) {
         ev_io_stop (loop, &c->watcher);
@@ -671,7 +673,7 @@ main (int argc, char *argv[])
         if (r->data == NULL)
             FAIL ("Cannot allocate buffer: %s", strerror (errno));
 
-        start_request(r);
+        start_request (r);
     }
 
     /* Start watching events on src and dst handles. */
