@@ -1,6 +1,6 @@
 (* hey emacs, this is OCaml code: -*- tuareg -*- *)
 (* nbd client library in userspace: generate the C API and documentation
- * Copyright (C) 2013-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@ open API
 open Utils
 
 type closure_style = Direct | AddressOf | Pointer
+type parens_style = NoParens | ParensSameLine | ParensNewLineWithIndent of int
 
 let generate_lib_libnbd_syms () =
   generate_header HashStyle;
@@ -68,7 +69,8 @@ let errcode_of_ret =
   function
   | RBool | RErr | RFd | RInt | RInt64 | RCookie | RSizeT -> Some "-1"
   | RStaticString | RString -> Some "NULL"
-  | RUInt | RUIntPtr | REnum _ | RFlags _ -> None (* errors not possible *)
+  | RUInt | RUIntPtr | RUInt64 | REnum _
+  | RFlags _ -> None (* errors not possible *)
 
 let type_of_ret =
   function
@@ -79,6 +81,7 @@ let type_of_ret =
   | RString -> "char *"
   | RUInt -> "unsigned"
   | RUIntPtr -> "uintptr_t"
+  | RUInt64 -> "uint64_t"
   | RFlags _ -> "uint32_t"
 
 let name_of_arg =
@@ -132,15 +135,23 @@ let arg_attr_nonnull =
 
 let optarg_attr_nonnull (OClosure _ | OFlags _) = [ false ]
 
-let rec print_arg_list ?(wrap = false) ?maxcol ?handle ?types ?(parens = true)
-          ?closure_style args optargs =
-  if parens then pr "(";
+let rec print_arg_list ?(wrap = false) ?maxcol ?handle ?types
+          ?(parens = ParensSameLine) ?closure_style args optargs =
+  (match parens with
+   | NoParens -> ()
+   | ParensSameLine -> pr "("
+   | ParensNewLineWithIndent indent -> pr "(\n%s" (spaces (indent + 2))
+  );
   if wrap then
     pr_wrap ?maxcol ','
       (fun () -> print_arg_list' ?handle ?types ?closure_style args optargs)
   else
     print_arg_list' ?handle ?types ?closure_style args optargs;
-  if parens then pr ")"
+  (match parens with
+   | NoParens -> ()
+   | ParensSameLine -> pr ")"
+   | ParensNewLineWithIndent indent -> pr "\n%s)" (spaces indent)
+  )
 
 and print_arg_list' ?(handle = false) ?(types = true) ?(closure_style = Direct)
           args optargs =
@@ -235,8 +246,12 @@ and print_arg_list' ?(handle = false) ?(types = true) ?(closure_style = Direct)
   ) optargs
 
 let print_call ?wrap ?maxcol ?closure_style name args optargs ret =
-  pr "%s nbd_%s " (type_of_ret ret) name;
-  print_arg_list ~handle:true ?wrap ?maxcol ?closure_style args optargs
+  pr "%s " (type_of_ret ret);
+  let designator_column = output_column () in
+  pr "nbd_%s " name;
+  print_arg_list ~handle:true ?wrap ?maxcol
+      ~parens:(ParensNewLineWithIndent designator_column) ?closure_style args
+      optargs
 
 let print_fndecl ?wrap ?closure_style name args optargs ret =
   pr "extern ";
@@ -248,15 +263,15 @@ let print_fndecl ?wrap ?closure_style name args optargs ret =
    *)
   (match ret with
    | RString ->
-      pr "\n    LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(__builtin_free)";
+      pr "\n    LIBNBD_ATTRIBUTE_ALLOC_DEALLOC (__builtin_free)";
    | _ -> ()
   );
 
   (* Output __attribute__((nonnull)) for the function parameters:
    * eg. struct nbd_handle *, int, char *
    *     => [ true, false, true ]
-   *     => LIBNBD_ATTRIBUTE_NONNULL(1,3)
-   *     => __attribute__((nonnull(1,3)))
+   *     => LIBNBD_ATTRIBUTE_NONNULL (1, 3)
+   *     => __attribute__ ((nonnull (1, 3)))
    *)
   let nns : bool list =
     [ true ] (* struct nbd_handle * *)
@@ -265,7 +280,7 @@ let print_fndecl ?wrap ?closure_style name args optargs ret =
   let nns = List.mapi (fun i b -> (i+1, b)) nns in
   let nns = filter_map (fun (i, b) -> if b then Some i else None) nns in
   let nns : string list = List.map string_of_int nns in
-  pr "\n    LIBNBD_ATTRIBUTE_NONNULL(%s);\n" (String.concat "," nns)
+  pr "\n    LIBNBD_ATTRIBUTE_NONNULL (%s);\n" (String.concat ", " nns)
 
 let rec print_cbarg_list ?(wrap = false) ?maxcol ?types ?(parens = true)
           cbargs =
@@ -398,22 +413,23 @@ let generate_include_libnbd_h () =
   pr "extern \"C\" {\n";
   pr "#endif\n";
   pr "\n";
-  pr "#if defined(__GNUC__)\n";
+  pr "#if defined (__GNUC__)\n";
   pr "#define LIBNBD_GCC_VERSION \\\n";
   pr "    (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)\n";
   pr "#endif\n";
   pr "\n";
   pr "#ifndef LIBNBD_ATTRIBUTE_NONNULL\n";
-  pr "#if defined(__GNUC__) && LIBNBD_GCC_VERSION >= 120000 /* gcc >= 12.0 */\n";
-  pr "#define LIBNBD_ATTRIBUTE_NONNULL(...) __attribute__((__nonnull__(__VA_ARGS__)))\n";
+  pr "#if defined (__GNUC__) && LIBNBD_GCC_VERSION >= 120000 /* gcc >= 12.0 */\n";
+  pr "#define LIBNBD_ATTRIBUTE_NONNULL(...) \\\n";
+  pr "  __attribute__ ((__nonnull__ (__VA_ARGS__)))\n";
   pr "#else\n";
   pr "#define LIBNBD_ATTRIBUTE_NONNULL(...)\n";
   pr "#endif\n";
   pr "#endif /* ! defined LIBNBD_ATTRIBUTE_NONNULL */\n";
   pr "\n";
-  pr "#if defined(__GNUC__) && LIBNBD_GCC_VERSION >= 120000 /* gcc >= 12.0 */\n";
+  pr "#if defined (__GNUC__) && LIBNBD_GCC_VERSION >= 120000 /* gcc >= 12.0 */\n";
   pr "#define LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(fn) \\\n";
-  pr "  __attribute__((__malloc__, __malloc__(fn, 1)))\n";
+  pr "  __attribute__ ((__malloc__, __malloc__ (fn, 1)))\n";
   pr "#else\n";
   pr "#define LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(fn)\n";
   pr "#endif\n";
@@ -452,7 +468,7 @@ let generate_include_libnbd_h () =
   pr "#define LIBNBD_HAVE_NBD_CLOSE 1\n";
   pr "\n";
   pr "extern struct nbd_handle *nbd_create (void)\n";
-  pr "    LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(nbd_close);\n";
+  pr "    LIBNBD_ATTRIBUTE_ALLOC_DEALLOC (nbd_close);\n";
   pr "#define LIBNBD_HAVE_NBD_CREATE 1\n";
   pr "\n";
   pr "extern const char *nbd_get_error (void);\n";
@@ -517,21 +533,27 @@ let generate_lib_api_c () =
       pr "{\n";
       pr "  const enum state state = get_public_state (h);\n";
       pr "\n";
+      let or_nl_indent = " ||\n        " in
       let tests =
         List.map (
           function
           | Created -> "nbd_internal_is_state_created (state)"
           | Connecting -> "nbd_internal_is_state_connecting (state)"
           | Negotiating -> "nbd_internal_is_state_negotiating (state)"
-          | Connected -> "nbd_internal_is_state_ready (state) || nbd_internal_is_state_processing (state)"
+          | Connected ->
+              "nbd_internal_is_state_ready (state)" ^
+              or_nl_indent ^
+              "nbd_internal_is_state_processing (state)"
           | Closed -> "nbd_internal_is_state_closed (state)"
           | Dead -> "nbd_internal_is_state_dead (state)"
         ) permitted_states in
-      pr "  if (!(%s)) {\n" (String.concat " ||\n        " tests);
+      pr "  if (!(%s)) {\n" (String.concat or_nl_indent tests);
       pr "    set_error (nbd_internal_is_state_created (state) ? ENOTCONN : EINVAL,\n";
       pr "               \"invalid state: %%s: the handle must be %%s\",\n";
       pr "               nbd_internal_state_short_string (state),\n";
-      pr "               \"%s\");\n" (permitted_state_text permitted_states);
+      pr "               \"";
+      pr_wrap_cstr (fun () -> pr "%s" (permitted_state_text permitted_states));
+      pr "\");\n";
       pr "    return false;\n";
       pr "  }\n";
       pr "  return true;\n";
@@ -544,9 +566,12 @@ let generate_lib_api_c () =
     let ret_c_type = type_of_ret ret and errcode = errcode_of_ret ret in
     pr "%s\n" ret_c_type;
     pr "nbd_%s " name;
-    print_arg_list ~wrap:true ~handle:true args optargs;
+    print_arg_list ~wrap:true ~handle:true ~parens:(ParensNewLineWithIndent 0)
+      args optargs;
     pr "\n";
     pr "{\n";
+    if permitted_states <> [] then
+      pr "  bool p;\n";
     pr "  %s ret;\n" ret_c_type;
     pr "\n";
     if may_set_error then (
@@ -577,7 +602,8 @@ let generate_lib_api_c () =
       let value = match errcode with
         | Some value -> value
         | None -> assert false in
-      pr "  if (unlikely (!%s_in_permitted_state (h))) {\n" name;
+      pr "  p = %s_in_permitted_state (h);\n" name;
+      pr "  if (unlikely (!p)) {\n";
       pr "    ret = %s;\n" value;
       pr "    goto out;\n";
       pr "  }\n";
@@ -600,7 +626,7 @@ let generate_lib_api_c () =
            sprintf "0x%x" !v
         | None -> "LIBNBD_" ^ flag_prefix ^ "_MASK" in
       let guard = match guard with
-        | Some value -> " && " ^ value
+        | Some value -> " &&\n      " ^ value
         | None -> "" in
       pr "  if (unlikely ((%s & ~%s) != 0)%s) {\n" n mask guard;
       pr "    set_error (EINVAL, \"%%s: invalid value for flag: 0x%%x\",\n";
@@ -718,63 +744,88 @@ let generate_lib_api_c () =
       | Int64 _ | SizeT _
       | SockAddrAndLen _ | UInt _ | UInt32 _ | UInt64 _ | UIntPtr _ -> ()
     ) args;
-    if may_set_error then
-      pr "    debug (h, \"enter:"
-    else (
-      pr "    debug_direct (h, \"nbd_%s\", \"enter:" name
-    );
-    List.iter (
-      function
-      | Bool n -> pr " %s=%%s" n
-      | BytesOut (n, count)
-      | BytesPersistOut (n, count) -> pr " %s=<buf> %s=%%zu" n count
-      | BytesIn (n, count)
-      | BytesPersistIn (n, count) ->
-         pr " %s=\\\"%%s\\\" %s=%%zu" n count
-      | Closure { cbname } -> pr " %s=<fun>" cbname
-      | Enum (n, _) -> pr " %s=%%d" n
-      | Flags (n, _) -> pr " %s=0x%%x" n
-      | Fd n | Int n -> pr " %s=%%d" n
-      | Int64 n -> pr " %s=%%\" PRIi64 \"" n
-      | SizeT n -> pr " %s=%%zu" n
-      | SockAddrAndLen (n, len) -> pr " %s=<sockaddr> %s=%%d" n len
-      | Path n
-      | String n -> pr " %s=%%s" n
-      | StringList n -> pr " %s=%%s" n
-      | UInt n -> pr " %s=%%u" n
-      | UInt32 n -> pr " %s=%%\" PRIu32 \"" n
-      | UInt64 n -> pr " %s=%%\" PRIu64 \"" n
-      | UIntPtr n -> pr " %s=%%\" PRIuPTR \"" n
-    ) args;
-    List.iter (
-      function
-      | OClosure { cbname } -> pr " %s=%%s" cbname
-      | OFlags (n, _, _) -> pr " %s=0x%%x" n
-    ) optargs;
+    let indent =
+      if may_set_error then (
+        pr "    debug (h,\n";
+        spaces 11
+      )
+      else (
+        pr "    debug_direct (h, \"nbd_%s\",\n" name;
+        spaces 18
+      ) in
+    pr "%s\"" indent;
+    let print_format_string () =
+      pr "enter:";
+      List.iter (
+        function
+        | Bool n -> pr " %s=%%s" n
+        | BytesOut (n, count)
+        | BytesPersistOut (n, count) -> pr " %s=<buf> %s=%%zu" n count
+        | BytesIn (n, count)
+        | BytesPersistIn (n, count) ->
+           pr " %s=\\\"%%s\\\" %s=%%zu" n count
+        | Closure { cbname } -> pr " %s=%%s" cbname
+        | Enum (n, _) -> pr " %s=%%d" n
+        | Flags (n, _) -> pr " %s=0x%%x" n
+        | Fd n | Int n -> pr " %s=%%d" n
+        | Int64 n -> pr " %s=%%\"PRIi64\"" n
+        | SizeT n -> pr " %s=%%zu" n
+        | SockAddrAndLen (n, len) -> pr " %s=<sockaddr> %s=%%d" n len
+        | Path n
+        | String n -> pr " %s=%%s" n
+        | StringList n -> pr " %s=%%s" n
+        | UInt n -> pr " %s=%%u" n
+        | UInt32 n -> pr " %s=%%\"PRIu32\"" n
+        | UInt64 n -> pr " %s=%%\"PRIu64\"" n
+        | UIntPtr n -> pr " %s=%%\"PRIuPTR\"" n
+      ) args;
+      List.iter (
+        function
+        | OClosure { cbname } -> pr " %s=%%s" cbname
+        | OFlags (n, _, _) -> pr " %s=0x%%x" n
+      ) optargs
+    in
+    pr_wrap_cstr print_format_string;
     pr "\"";
-    List.iter (
-      function
-      | Bool n -> pr ", %s ? \"true\" : \"false\"" n
-      | BytesOut (n, count)
-      | BytesPersistOut (n, count) -> pr ", %s" count
-      | BytesIn (n, count)
-      | BytesPersistIn (n, count) ->
-         pr ", %s_printable ? %s_printable : \"\", %s" n n count
-      | Closure { cbname } -> ()
-      | Enum (n, _) -> pr ", %s" n
-      | Flags (n, _) -> pr ", %s" n
-      | Fd n | Int n | Int64 n | SizeT n -> pr ", %s" n
-      | SockAddrAndLen (_, len) -> pr ", (int) %s" len
-      | Path n | String n | StringList n ->
-         pr ", %s_printable ? %s_printable : \"\"" n n
-      | UInt n | UInt32 n | UInt64 n | UIntPtr n -> pr ", %s" n
-    ) args;
-    List.iter (
-      function
-      | OClosure { cbname } ->
-         pr ", CALLBACK_IS_NULL (%s_callback) ? \"<fun>\" : \"NULL\"" cbname
-      | OFlags (n, _, _) -> pr ", %s" n
-    ) optargs;
+    let num_args = List.length args
+    and num_optargs = List.length optargs in
+    let num_allargs = num_args + num_optargs in
+    if num_allargs > 0 then
+      pr ",\n%s" indent;
+    let print_args () =
+      List.iteri (
+        fun i arg ->
+          (match arg with
+           | Bool n -> pr "%s ? \"true\" : \"false\"" n
+           | BytesOut (n, count)
+           | BytesPersistOut (n, count) -> pr "%s" count
+           | BytesIn (n, count)
+           | BytesPersistIn (n, count) ->
+              pr "%s_printable ? %s_printable : \"\", %s" n n count
+           | Closure _ -> pr "\"<fun>\""
+           | Enum (n, _) -> pr "%s" n
+           | Flags (n, _) -> pr "%s" n
+           | Fd n | Int n | Int64 n | SizeT n -> pr "%s" n
+           | SockAddrAndLen (_, len) -> pr "(int) %s" len
+           | Path n | String n | StringList n ->
+              pr "%s_printable ? %s_printable : \"\"" n n
+           | UInt n | UInt32 n | UInt64 n | UIntPtr n -> pr "%s" n
+          );
+          if i < num_allargs - 1 then
+            pr ", "
+      ) args;
+      List.iteri (
+        fun i optarg ->
+          (match optarg with
+           | OClosure { cbname } ->
+              pr "CALLBACK_IS_NULL (%s_callback) ? \"<fun>\" : \"NULL\"" cbname
+           | OFlags (n, _, _) -> pr "%s" n
+          );
+          if num_args + i < num_allargs - 1 then
+            pr ", "
+      ) optargs
+    in
+    pr_wrap ',' print_args;
     pr ");\n";
     List.iter (
       function
@@ -807,20 +858,27 @@ let generate_lib_api_c () =
           pr "          nbd_internal_printable_string (ret);\n"
        | _ -> ()
       );
-      pr "      debug (h, \"leave: ret=\" "
+      pr "      debug ("
     )
-    else (
-      pr "    debug_direct (h, \"nbd_%s\", \"leave: ret=\" " name
-    );
-    (match ret with
-     | RBool | RErr | RFd | RInt | REnum _ -> pr "\"%%d\", ret"
-     | RInt64 | RCookie -> pr "\"%%\" PRIi64 \"\", ret"
-     | RSizeT -> pr "\"%%zd\", ret"
-     | RString -> pr "\"%%s\", ret_printable ? ret_printable : \"\""
-     | RStaticString -> pr "\"%%s\", ret"
-     | RUInt | RFlags _ -> pr "\"%%u\", ret"
-     | RUIntPtr -> pr "\"%%\" PRIuPTR, ret"
-    );
+    else
+      pr "    debug_direct (";
+    let print_args () =
+      if may_set_error then
+        pr "h, \"leave: ret="
+      else
+        pr "h, \"nbd_%s\", \"leave: ret=" name;
+      (match ret with
+       | RBool | RErr | RFd | RInt | REnum _ -> pr "%%d\", ret"
+       | RInt64 | RCookie -> pr "%%\" PRIi64, ret"
+       | RSizeT -> pr "%%zd\", ret"
+       | RString -> pr "%%s\", ret_printable ? ret_printable : \"\""
+       | RStaticString -> pr "%%s\", ret"
+       | RUInt | RFlags _ -> pr "%%u\", ret"
+       | RUIntPtr -> pr "%%\" PRIuPTR, ret"
+       | RUInt64 -> pr "%%\" PRIu64, ret"
+      )
+    in
+    pr_wrap ',' print_args;
     pr ");\n";
     if may_set_error then (
       (match ret with
@@ -989,6 +1047,8 @@ let generate_docs_nbd_pod name { args; optargs; ret;
       pr "This call returns a bitmask.\n";
    | RUIntPtr ->
       pr "This call returns a C<uintptr_t>.\n";
+   | RUInt64 ->
+      pr "This call returns a counter.\n";
    | REnum { enum_prefix } ->
       pr "This call returns one of the LIBNBD_%s_* values.\n" enum_prefix;
    | RFlags { flag_prefix } ->
@@ -1095,5 +1155,5 @@ Richard W.M. Jones
 
 =head1 COPYRIGHT
 
-Copyright (C) 2019-2021 Red Hat Inc.
+Copyright Red Hat
 "
