@@ -1,6 +1,6 @@
 (* hey emacs, this is OCaml code: -*- tuareg -*- *)
 (* nbd client library in userspace: generator
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -69,6 +69,7 @@ and ocaml_ret_to_string = function
   | RSizeT -> "int"
   | RString -> "string"
   | RUInt | RUIntPtr -> "int"
+  | RUInt64 -> "int64"
   | REnum { enum_prefix } -> enum_prefix ^ ".t"
   | RFlags { flag_prefix } -> flag_prefix ^ ".t list"
 
@@ -447,7 +448,7 @@ let print_ocaml_flag_val { flag_prefix; flags } =
   pr "      if (bit > 31)\n";
   pr "        caml_invalid_argument (\"bitmask value out of range\");\n";
   pr "      else\n";
-  pr "        r |= 1 << bit;\n";
+  pr "        r |= 1u << bit;\n";
   pr "    }\n";
   pr "  }\n";
   pr "\n";
@@ -523,8 +524,13 @@ let print_ocaml_closure_wrapper { cbname; cbargs } =
   List.iter (
     function
     | CBArrayAndLen (UInt32 n, count) ->
-       pr "  %sv = nbd_internal_ocaml_alloc_int64_from_uint32_array (%s, %s);\n"
-         n n count;
+       pr "  %sv = " n;
+       let fncol = output_column () in
+       let indent = spaces fncol in
+       pr "nbd_internal_ocaml_alloc_i64_from_u32_array (\n";
+       pr "%s  %s,\n" indent n;
+       pr "%s  %s\n" indent count;
+       pr "%s);\n" indent
     | CBBytesIn (n, len) ->
        pr "  %sv = caml_alloc_initialized_string (%s, %s);\n" n len n
     | CBInt n | CBUInt n ->
@@ -627,7 +633,9 @@ let print_ocaml_binding (name, { args; optargs; ret }) =
 
   pr "  struct nbd_handle *h = NBD_val (hv);\n";
   pr "  if (h == NULL)\n";
-  pr "    nbd_internal_ocaml_raise_closed (\"NBD.%s\");\n" name;
+  pr "    nbd_internal_ocaml_raise_closed (\n";
+  pr "      \"NBD.%s\"\n" name;
+  pr "    );\n";
   pr "\n";
 
   List.iter (
@@ -702,12 +710,12 @@ let print_ocaml_binding (name, { args; optargs; ret }) =
        pr "  size_t %s = Int_val (%sv);\n" n n
     | SockAddrAndLen (n, len) ->
        pr "  struct sockaddr_storage %s_storage;\n" n;
-       pr "  struct sockaddr *%s = (struct sockaddr *) &%s_storage;\n" n n;
+       pr "  struct sockaddr *%s = (struct sockaddr *)&%s_storage;\n" n n;
        pr "  socklen_t %s;\n" len;
        pr "  nbd_internal_unix_sockaddr_to_sa (%sv, &%s_storage, &%s);\n"
          n n len
     | StringList n ->
-       pr "  char **%s = (char **) nbd_internal_ocaml_string_list (%sv);\n" n n
+       pr "  char **%s = (char **)nbd_internal_ocaml_string_list (%sv);\n" n n
     | UInt n | UIntPtr n ->
        pr "  unsigned %s = Int_val (%sv);\n" n n
     | UInt32 n ->
@@ -754,7 +762,7 @@ let print_ocaml_binding (name, { args; optargs; ret }) =
    | RFd | RInt | RSizeT | RUInt | RUIntPtr -> pr "  rv = Val_int (r);\n"
    | REnum { enum_prefix } -> pr "  rv = Val_%s (r);\n" enum_prefix
    | RFlags { flag_prefix } -> pr "  rv = Val_%s (r);\n" flag_prefix
-   | RInt64 | RCookie -> pr "  rv = caml_copy_int64 (r);\n"
+   | RInt64 | RCookie | RUInt64 -> pr "  rv = caml_copy_int64 (r);\n"
    | RStaticString -> pr "  rv = caml_copy_string (r);\n"
    | RString ->
       pr "  rv = caml_copy_string (r);\n";
@@ -790,17 +798,21 @@ let print_ocaml_binding (name, { args; optargs; ret }) =
   pr "}\n";
   pr "\n";
 
-  if num_params args optargs > 5 then (
+  let num_args = num_params args optargs in
+  if num_args > 5 then (
     pr "/* Byte-code compat function because this method has > 5 parameters.\n";
     pr " */\n";
     pr "value\n";
     pr "nbd_internal_ocaml_nbd_%s_byte (value *argv, int argn)\n" name;
     pr "{\n";
     pr "  return nbd_internal_ocaml_nbd_%s (" name;
-    for i = 0 to num_params args optargs - 1 do
-      if i > 0 then pr ", ";
-      pr "argv[%d]" i
-    done;
+    let print_args () =
+      for i = 0 to num_args - 1 do
+        if i > 0 then pr ", ";
+        pr "argv[%d]" i
+      done
+    in
+    pr_wrap ',' print_args;
     pr ");\n";
     pr "}\n";
     pr "\n"
