@@ -1,6 +1,6 @@
 (* hey emacs, this is OCaml code: -*- tuareg -*- *)
 (* nbd client library in userspace: state machine definition
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -878,6 +878,13 @@ and structured_reply_state_machine = [
     external_events = [];
   };
 
+  State {
+    default_state with
+    name = "RESYNC";
+    comment = "Ignore payload of an unexpected structured reply";
+    external_events = [];
+  };
+
   State {
     default_state with
     name = "FINISH";
