@@ -1,5 +1,5 @@
 /* nbd client library in userspace: state machine
- * Copyright (C) 2013-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,25 +22,49 @@ STATE_MACHINE {
  REPLY.SIMPLE_REPLY.START:
   struct command *cmd = h->reply_cmd;
   uint32_t error;
-  uint64_t cookie;
 
   error = be32toh (h->sbuf.simple_reply.error);
-  cookie = be64toh (h->sbuf.simple_reply.handle);
 
-  assert (cmd);
-  assert (cmd->cookie == cookie);
-
-  if (cmd->type == NBD_CMD_READ && h->structured_replies) {
-    set_error (0, "server sent unexpected simple reply for read");
-    SET_NEXT_STATE(%.DEAD);
+  if (cmd == NULL) {
+    /* Unexpected reply.  If error was set or we have structured
+     * replies, we know there should be no payload, so the next byte
+     * on the wire (if any) will be another reply, and we can let
+     * FINISH_COMMAND diagnose/ignore the server bug.  If not, we lack
+     * context to know whether the server thinks it was responding to
+     * NBD_CMD_READ, so it is safer to move to DEAD now than to risk
+     * consuming a server's potential data payload as a reply stream
+     * (even though we would be likely to produce a magic number
+     * mismatch on the next pass that would also move us to DEAD).
+     */
+    if (error || h->structured_replies)
+      SET_NEXT_STATE (%^FINISH_COMMAND);
+    else {
+      uint64_t cookie = be64toh (h->sbuf.simple_reply.cookie);
+      SET_NEXT_STATE (%.DEAD);
+      set_error (EPROTO,
+                 "no matching cookie %" PRIu64 " found for server reply, "
+                 "this is probably a server bug", cookie);
+    }
     return 0;
   }
 
-  cmd->error = nbd_internal_errno_of_nbd_error (error);
-  if (cmd->error == 0 && cmd->type == NBD_CMD_READ) {
+  /* Although a server with structured replies negotiated is in error
+   * for using a simple reply to NBD_CMD_READ, we can cope with the
+   * packet, but diagnose it by failing the read with EPROTO.
+   */
+  if (cmd->type == NBD_CMD_READ && h->structured_replies) {
+    debug (h, "server sent unexpected simple reply for read");
+    if (cmd->error == 0)
+      cmd->error = EPROTO;
+  }
+
+  error = nbd_internal_errno_of_nbd_error (error);
+  if (cmd->error == 0)
+    cmd->error = error;
+  if (error == 0 && cmd->type == NBD_CMD_READ) {
     h->rbuf = cmd->data;
     h->rlen = cmd->count;
-    cmd->data_seen = cmd->count;
+    cmd->data_seen += cmd->count;
     SET_NEXT_STATE (%RECV_READ_PAYLOAD);
   }
   else {
@@ -61,9 +85,8 @@ STATE_MACHINE {
     /* guaranteed by START */
     assert (cmd);
     if (CALLBACK_IS_NOT_NULL (cmd->cb.fn.chunk)) {
-      int error = 0;
+      int error = cmd->error;
 
-      assert (cmd->error == 0);
       if (CALL_CALLBACK (cmd->cb.fn.chunk,
                          cmd->data, cmd->count,
                          cmd->offset, LIBNBD_READ_DATA,
