@@ -1,5 +1,5 @@
 /* nbd client library in userspace: state machine
- * Copyright (C) 2013-2019 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,26 +25,28 @@
  * until POLLIN, we instead track where in the state machine we left
  * off, then return to READY to actually block. Then, on entry to
  * REPLY.START, we can tell if this is the start of a new reply (rlen
- * is 0, stay put), a continuation of the preamble (reply_cmd is NULL,
- * resume with RECV_REPLY), or a continuation from any other location
- * (reply_cmd contains the state to jump to).
+ * is 0, stay put), a continuation of the preamble (reply_state is
+ * STATE_START, resume with RECV_REPLY), or a continuation from any
+ * other location (reply_state contains the state to jump to).
  */
 
 static void
 save_reply_state (struct nbd_handle *h)
 {
-  assert (h->reply_cmd);
   assert (h->rlen);
-  h->reply_cmd->state = get_next_state (h);
+  assert (h->reply_state == STATE_START);
+  h->reply_state = get_next_state (h);
+  assert (h->reply_state != STATE_START);
 }
 
 STATE_MACHINE {
  REPLY.START:
   /* If rlen is non-zero, we are resuming an earlier reply cycle. */
   if (h->rlen > 0) {
-    if (h->reply_cmd) {
-      assert (nbd_internal_is_state_processing (h->reply_cmd->state));
-      SET_NEXT_STATE (h->reply_cmd->state);
+    if (h->reply_state != STATE_START) {
+      assert (nbd_internal_is_state_processing (h->reply_state));
+      SET_NEXT_STATE (h->reply_state);
+      h->reply_state = STATE_START;
     }
     else
       SET_NEXT_STATE (%RECV_REPLY);
@@ -94,7 +96,8 @@ STATE_MACHINE {
   nbd_internal_hexdump (h->rbuf, r, stderr);
 #endif
 
-  h->rbuf += r;
+  h->bytes_received += r;
+  h->rbuf = (char *)h->rbuf + r;
   h->rlen -= r;
   SET_NEXT_STATE (%RECV_REPLY);
   return 0;
@@ -120,31 +123,29 @@ STATE_MACHINE {
     SET_NEXT_STATE (%STRUCTURED_REPLY.START);
   }
   else {
-    SET_NEXT_STATE (%.DEAD); /* We've probably lost synchronization. */
-    set_error (0, "invalid reply magic");
+    /* We've probably lost synchronization. */
+    SET_NEXT_STATE (%.DEAD);
+    set_error (0, "invalid reply magic 0x%" PRIx32, magic);
+#if 0 /* uncomment to see desynchronized data */
+    nbd_internal_hexdump (&h->sbuf.simple_reply,
+                          sizeof (h->sbuf.simple_reply),
+                          stderr);
+#endif
     return 0;
   }
 
   /* NB: This works for both simple and structured replies because the
    * handle (our cookie) is stored at the same offset.
    */
-  cookie = be64toh (h->sbuf.simple_reply.handle);
-  /* Find the command amongst the commands in flight. */
+  h->chunks_received++;
+  cookie = be64toh (h->sbuf.simple_reply.cookie);
+  /* Find the command amongst the commands in flight. If the server sends
+   * a reply for an unknown cookie, FINISH will diagnose that later.
+   */
   for (cmd = h->cmds_in_flight; cmd != NULL; cmd = cmd->next) {
     if (cmd->cookie == cookie)
       break;
   }
-  if (cmd == NULL) {
-    /* An unexpected structured reply could be skipped, since it
-     * includes a length; similarly an unexpected simple reply can be
-     * skipped if we assume it was not a read. However, it's more
-     * likely we've lost synchronization with the server.
-     */
-    SET_NEXT_STATE (%.DEAD);
-    set_error (0, "no matching cookie found for server reply, "
-               "this is probably a bug in the server");
-    return 0;
-  }
   h->reply_cmd = cmd;
   return 0;
 
@@ -156,7 +157,7 @@ STATE_MACHINE {
   /* NB: This works for both simple and structured replies because the
    * handle (our cookie) is stored at the same offset.
    */
-  cookie = be64toh (h->sbuf.simple_reply.handle);
+  cookie = be64toh (h->sbuf.simple_reply.cookie);
   /* Find the command amongst the commands in flight. */
   for (cmd = h->cmds_in_flight, prev_cmd = NULL;
        cmd != NULL;
@@ -164,10 +165,16 @@ STATE_MACHINE {
     if (cmd->cookie == cookie)
       break;
   }
-  assert (cmd != NULL);
   assert (h->reply_cmd == cmd);
-  h->reply_cmd = NULL;
+  if (cmd == NULL) {
+    debug (h, "skipped reply for unexpected cookie %" PRIu64
+           ", this is probably a bug in the server", cookie);
+    SET_NEXT_STATE (%.READY);
+    return 0;
+  }
+
   retire = cmd->type == NBD_CMD_DISC;
+  h->reply_cmd = NULL;
 
   /* Notify the user */
   if (CALLBACK_IS_NOT_NULL (cmd->cb.completion)) {
