@@ -1,6 +1,6 @@
 (* hey emacs, this is OCaml code: -*- tuareg -*- *)
 (* nbd client library in userspace: utilities
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,7 @@ val char_mem : char -> string -> bool
 val span : string -> string -> int
 val cspan : string -> string -> int
 val quote : string -> string
+val spaces : int -> string
 val files_equal : string -> string -> bool
 
 val generate_header : ?extra_sources:string list -> comment_style -> unit
@@ -52,6 +53,8 @@ val generate_header : ?extra_sources:string list -> comment_style -> unit
 val output_to : string -> (unit -> 'a) -> unit
 val pr : ('a, unit, string, unit) format4 -> 'a
 val pr_wrap : ?maxcol:int -> char -> (unit -> 'a) -> unit
+val pr_wrap_cstr : ?maxcol:int -> (unit -> 'a) -> unit
+val pr_wrap_c_comment : ?maxcol:int -> (unit -> 'a) -> unit
 val output_lineno : unit -> int
 val output_column : unit -> int
 
