@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbd client library in userspace
-# Copyright (C) 2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@ set -e
 set -x
 
 requires nbdkit --version
+requires nbdkit -U - null --run 'test "$uri" != ""'
 requires tr --version
 
 out=info-base-allocation-large.out
@@ -35,11 +36,11 @@ nbdkit -U - data data='1 @131072 2 @6442450944 3' size=8G \
 
 cat $out
 
-if [ "$(tr -s ' ' < $out)" != " 0 32768 0 allocated
+if [ "$(tr -s ' ' < $out)" != " 0 32768 0 data
  32768 98304 3 hole,zero
- 131072 32768 0 allocated
+ 131072 32768 0 data
  163840 6442287104 3 hole,zero
-6442450944 32768 0 allocated
+6442450944 32768 0 data
 6442483712 2147450880 3 hole,zero" ]; then
     echo "$0: unexpected output from nbdinfo --map"
     exit 1
