@@ -1,5 +1,5 @@
 /* nbd client library in userspace: internal definitions
- * Copyright (C) 2013-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -46,7 +46,7 @@
  * debug and error handling code out of hot paths, making the hot path
  * into common functions use less instruction cache.
  */
-#if defined(__GNUC__)
+#if defined (__GNUC__)
 #define unlikely(x) __builtin_expect (!!(x), 0)
 #define if_debug(h) if (unlikely ((h)->debug))
 #else
@@ -71,7 +71,7 @@ struct meta_context {
   char *name;                   /* Name of meta context. */
   uint32_t context_id;          /* Context ID negotiated with the server. */
 };
-DEFINE_VECTOR_TYPE(meta_vector, struct meta_context);
+DEFINE_VECTOR_TYPE (meta_vector, struct meta_context);
 
 struct export {
   char *name;
@@ -101,6 +101,7 @@ struct nbd_handle {
   _Atomic uintptr_t private_data;
 
   char *export_name;            /* Export name, never NULL. */
+  char *sact_name;              /* Socket activation name, can be NULL. */
 
   /* TLS settings. */
   int tls;                      /* 0 = disable, 1 = enable, 2 = require */
@@ -111,6 +112,7 @@ struct nbd_handle {
 
   /* Desired metadata contexts. */
   bool request_sr;
+  bool request_meta;
   string_vector request_meta_contexts;
 
   /* Allowed in URIs, see lib/uri.c. */
@@ -145,6 +147,8 @@ struct nbd_handle {
   uint32_t block_minimum;
   uint32_t block_preferred;
   uint32_t block_maximum;
+  /* Payload cap, always non-zero, based on block_maximum when feasible */
+  uint32_t payload_maximum;
 
   /* Server canonical name and description, or NULL if not advertised. */
   char *canonical_name;
@@ -158,6 +162,12 @@ struct nbd_handle {
 
   int64_t unique;               /* Used for generating cookie numbers. */
 
+  /* Traffic statistics. */
+  uint64_t bytes_sent;
+  uint64_t chunks_sent;
+  uint64_t bytes_received;
+  uint64_t chunks_received;
+
   /* For debugging. */
   bool debug;
   nbd_debug_callback debug_callback;
@@ -180,6 +190,7 @@ struct nbd_handle {
   bool structured_replies;      /* If we negotiated NBD_OPT_STRUCTURED_REPLY */
 
   /* Vector of negotiated metadata contexts. */
+  bool meta_valid;
   meta_vector meta_contexts;
 
   /* The socket or a wrapper if using GnuTLS. */
@@ -210,20 +221,20 @@ struct nbd_handle {
         struct {
           struct nbd_fixed_new_option_reply_server server;
           char str[NBD_MAX_STRING * 2 + 1]; /* name, description, NUL */
-        } __attribute__((packed)) server;
+        } __attribute__ ((packed)) server;
         struct nbd_fixed_new_option_reply_info_export export;
         struct nbd_fixed_new_option_reply_info_block_size block_size;
         struct {
           struct nbd_fixed_new_option_reply_info_name_or_desc info;
           char str[NBD_MAX_STRING];
-        } __attribute__((packed)) name_desc;
+        } __attribute__ ((packed)) name_desc;
         struct {
           struct nbd_fixed_new_option_reply_meta_context context;
           char str[NBD_MAX_STRING];
-        }  __attribute__((packed)) context;
+        }  __attribute__ ((packed)) context;
         char err_msg[NBD_MAX_STRING];
       } payload;
-    }  __attribute__((packed)) or;
+    }  __attribute__ ((packed)) or;
     struct nbd_export_name_option_reply export_name_reply;
     struct nbd_simple_reply simple_reply;
     struct {
@@ -235,9 +246,9 @@ struct nbd_handle {
           struct nbd_structured_reply_error error;
           char msg[NBD_MAX_STRING]; /* Common to all error types */
           uint64_t offset; /* Only used for NBD_REPLY_TYPE_ERROR_OFFSET */
-        } __attribute__((packed)) error;
+        } __attribute__ ((packed)) error;
       } payload;
-    }  __attribute__((packed)) sr;
+    }  __attribute__ ((packed)) sr;
     uint16_t gflags;
     uint32_t cflags;
     uint32_t len;
@@ -268,8 +279,8 @@ struct nbd_handle {
   /* When using systemd socket activation, this directory and socket
    * must be deleted, and the pid above must be killed.
    */
-  char *sa_tmpdir;
-  char *sa_sockpath;
+  char *sact_tmpdir;
+  char *sact_sockpath;
 
   /* When connecting to TCP ports, these fields are used. */
   char *hostname, *port;
@@ -278,6 +289,7 @@ struct nbd_handle {
   int connect_errno;
 
   /* When sending metadata contexts, this is used. */
+  string_vector querylist;
   size_t querynum;
 
   /* When receiving block status, this is used. */
@@ -308,8 +320,9 @@ struct nbd_handle {
   /* length (cmds_to_issue) + length (cmds_in_flight). */
   int in_flight;
 
-  /* Current command during a REPLY cycle */
+  /* Current command and POLLIN resumption point during a REPLY cycle */
   struct command *reply_cmd;
+  enum state reply_state;
 
   bool disconnect_request;      /* True if we've queued NBD_CMD_DISC */
   bool tls_shut_writes;         /* Used by lib/crypto.c to track disconnect. */
@@ -351,12 +364,23 @@ struct command {
   uint32_t count;
   void *data; /* Buffer for read/write */
   struct command_cb cb;
-  enum state state; /* State to resume with on next POLLIN */
   bool initialized; /* For read, true if getting a hole may skip memset */
   uint32_t data_seen; /* For read, cumulative size of data chunks seen */
   uint32_t error; /* Local errno value */
 };
 
+struct execvpe {
+  string_vector pathnames;
+
+  /* Note: "const_string_vector" is not a good type for "sh_argv" below. Even if
+   * we reserved enough space in a "const_string_vector",
+   * const_string_vector_append() would still not be async-signal-safe, due to
+   * the underlying const_string_vector_insert() calling assert().
+   */
+  char **sh_argv;
+  size_t num_sh_args;
+};
+
 /* Test if a callback is "null" or not, and set it to null. */
 #define CALLBACK_IS_NULL(cb)     ((cb).callback == NULL && (cb).free == NULL)
 #define CALLBACK_IS_NOT_NULL(cb) (! CALLBACK_IS_NULL ((cb)))
@@ -376,26 +400,28 @@ struct command {
 
 /* aio.c */
 extern void nbd_internal_retire_and_free_command (struct command *)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 
 /* connect.c */
 extern int nbd_internal_wait_until_connected (struct nbd_handle *h)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 
 /* crypto.c */
-extern struct socket *nbd_internal_crypto_create_session (struct nbd_handle *, struct socket *oldsock)
-  LIBNBD_ATTRIBUTE_NONNULL(1, 2);
+extern struct socket *
+  nbd_internal_crypto_create_session (struct nbd_handle *,
+                                      struct socket *oldsock)
+  LIBNBD_ATTRIBUTE_NONNULL (1, 2);
 extern bool nbd_internal_crypto_is_reading (struct nbd_handle *)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern int nbd_internal_crypto_handshake (struct nbd_handle *)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern void nbd_internal_crypto_debug_tls_enabled (struct nbd_handle *)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 
 /* debug.c */
 extern void nbd_internal_debug (struct nbd_handle *h, const char *context,
                                 const char *fs, ...)
-  LIBNBD_ATTRIBUTE_NONNULL(1, 3);
+  LIBNBD_ATTRIBUTE_NONNULL (1, 3);
 #define debug(h, fs, ...)                                   \
   do {                                                      \
     if_debug ((h))                                          \
@@ -409,10 +435,10 @@ extern void nbd_internal_debug (struct nbd_handle *h, const char *context,
 
 /* errors.c */
 extern void nbd_internal_set_error_context (const char *context)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern const char *nbd_internal_get_error_context (void);
 extern void nbd_internal_set_last_error (int errnum, char *error)
-  LIBNBD_ATTRIBUTE_NONNULL(2);
+  LIBNBD_ATTRIBUTE_NONNULL (2);
 #define set_error(errnum, fs, ...)                                      \
   do {                                                                  \
     int _e = (errnum);                                                  \
@@ -427,18 +453,22 @@ extern void nbd_internal_set_last_error (int errnum, char *error)
                      _context, ##__VA_ARGS__, strerror (_e));           \
     if (_r >= 0)                                                        \
       nbd_internal_set_last_error (_e, _errp);                          \
+    else                                                                \
+      nbd_internal_set_last_error (_e, fs /* best effort */);           \
   } while (0)
 
 /* flags.c */
 extern void nbd_internal_reset_size_and_flags (struct nbd_handle *h)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern int nbd_internal_set_size_and_flags (struct nbd_handle *h,
                                             uint64_t exportsize,
                                             uint16_t eflags)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern int nbd_internal_set_block_size (struct nbd_handle *h, uint32_t min,
                                         uint32_t pref, uint32_t max)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
+extern void nbd_internal_set_payload (struct nbd_handle *h)
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 
 /* is-state.c */
 extern bool nbd_internal_is_state_created (enum state state);
@@ -451,7 +481,7 @@ extern bool nbd_internal_is_state_closed (enum state state);
 
 /* opt.c */
 extern void nbd_internal_free_option (struct nbd_handle *h)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 
 /* protocol.c */
 extern int nbd_internal_errno_of_nbd_error (uint32_t error);
@@ -463,7 +493,7 @@ extern int64_t nbd_internal_command_common (struct nbd_handle *h,
                                             uint64_t offset, uint64_t count,
                                             int count_err, void *data,
                                             struct command_cb *cb)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 
 /* socket.c */
 struct socket *nbd_internal_socket_create (int fd);
@@ -471,12 +501,13 @@ struct socket *nbd_internal_socket_create (int fd);
 /* states.c */
 extern void nbd_internal_abort_commands (struct nbd_handle *h,
                                          struct command **list)
-  LIBNBD_ATTRIBUTE_NONNULL(1,2);
+  LIBNBD_ATTRIBUTE_NONNULL (1, 2);
 extern int nbd_internal_run (struct nbd_handle *h, enum external_event ev)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern const char *nbd_internal_state_short_string (enum state state);
 extern enum state_group nbd_internal_state_group (enum state state);
-extern enum state_group nbd_internal_state_group_parent (enum state_group group);
+extern enum state_group
+  nbd_internal_state_group_parent (enum state_group group);
 extern int nbd_internal_aio_get_direction (enum state state);
 
 #define set_next_state(h,next_state) ((h)->state) = (next_state)
@@ -485,22 +516,24 @@ extern int nbd_internal_aio_get_direction (enum state state);
 
 /* utils.c */
 extern void nbd_internal_hexdump (const void *data, size_t len, FILE *fp)
-  LIBNBD_ATTRIBUTE_NONNULL(1,3);
+  LIBNBD_ATTRIBUTE_NONNULL (1, 3);
 extern int nbd_internal_copy_string_list (string_vector *v, char **in)
-  LIBNBD_ATTRIBUTE_NONNULL(1,2);
+  LIBNBD_ATTRIBUTE_NONNULL (1, 2);
 extern int nbd_internal_set_argv (struct nbd_handle *h, char **argv)
-  LIBNBD_ATTRIBUTE_NONNULL(1,2);
+  LIBNBD_ATTRIBUTE_NONNULL (1, 2);
+extern int nbd_internal_set_querylist (struct nbd_handle *h, char **queries)
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern const char *nbd_internal_fork_safe_itoa (long v, char *buf, size_t len)
-  LIBNBD_ATTRIBUTE_NONNULL(2);
+  LIBNBD_ATTRIBUTE_NONNULL (2);
 extern void nbd_internal_fork_safe_perror (const char *s)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern char *nbd_internal_printable_buffer (const void *buf, size_t count)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern char *nbd_internal_printable_string (const char *str)
-  LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(free)
-  LIBNBD_ATTRIBUTE_NONNULL(1);
+  LIBNBD_ATTRIBUTE_ALLOC_DEALLOC (free)
+  LIBNBD_ATTRIBUTE_NONNULL (1);
 extern char *nbd_internal_printable_string_list (char **list)
-  LIBNBD_ATTRIBUTE_ALLOC_DEALLOC(free);
+  LIBNBD_ATTRIBUTE_ALLOC_DEALLOC (free);
 
 /* These are wrappers around socket(2) and socketpair(2).  They
  * always set SOCK_CLOEXEC.  nbd_internal_socket can set SOCK_NONBLOCK
@@ -510,6 +543,29 @@ extern int nbd_internal_socket (int domain, int type, int protocol,
                                 bool nonblock);
 extern int nbd_internal_socketpair (int domain, int type, int protocol,
                                     int *fds)
-  LIBNBD_ATTRIBUTE_NONNULL(4);
+  LIBNBD_ATTRIBUTE_NONNULL (4);
+
+extern void nbd_internal_fork_safe_assert (int result, const char *file,
+                                           long line, const char *func,
+                                           const char *assertion)
+  LIBNBD_ATTRIBUTE_NONNULL (2, 4, 5);
+
+#ifdef NDEBUG
+#define NBD_INTERNAL_FORK_SAFE_ASSERT(expression) ((void)0)
+#else
+#define NBD_INTERNAL_FORK_SAFE_ASSERT(expression)                        \
+  (nbd_internal_fork_safe_assert ((expression) != 0, __FILE__, __LINE__, \
+                                  __func__, #expression))
+#endif
+
+extern int nbd_internal_execvpe_init (struct execvpe *ctx, const char *file,
+                                      size_t num_args)
+  LIBNBD_ATTRIBUTE_NONNULL (1, 2);
+extern void nbd_internal_execvpe_uninit (struct execvpe *ctx)
+  LIBNBD_ATTRIBUTE_NONNULL (1);
+extern int nbd_internal_fork_safe_execvpe (struct execvpe *ctx,
+                                           const string_vector *argv,
+                                           char * const *envp)
+  LIBNBD_ATTRIBUTE_NONNULL (1, 2, 3);
 
 #endif /* LIBNBD_INTERNAL_H */
