@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -207,8 +207,15 @@ nbd_internal_command_common (struct nbd_handle *h,
 
   switch (type) {
     /* Commands which send or receive data are limited to MAX_REQUEST_SIZE. */
-  case NBD_CMD_READ:
   case NBD_CMD_WRITE:
+    if (h->strict & LIBNBD_STRICT_PAYLOAD && count > h->payload_maximum) {
+      set_error (ERANGE,
+                 "request too large: maximum payload size is %" PRIu32,
+                 h->payload_maximum);
+      goto err;
+    }
+    /* fallthrough */
+  case NBD_CMD_READ:
     if (count > MAX_REQUEST_SIZE) {
       set_error (ERANGE, "request too large: maximum request size is %d",
                  MAX_REQUEST_SIZE);
@@ -354,7 +361,7 @@ nbd_unlocked_aio_pwrite (struct nbd_handle *h, const void *buf,
 
   SET_CALLBACK_TO_NULL (*completion);
   return nbd_internal_command_common (h, flags, NBD_CMD_WRITE, offset, count,
-                                      ENOSPC, (void *) buf, &cb);
+                                      ENOSPC, (void *)buf, &cb);
 }
 
 int64_t
@@ -482,7 +489,7 @@ nbd_unlocked_aio_block_status (struct nbd_handle *h,
       return -1;
     }
 
-    if (h->meta_contexts == NULL) {
+    if (!h->meta_valid || h->meta_contexts.len == 0) {
       set_error (ENOTSUP, "did not negotiate any metadata contexts, "
                  "either you did not call nbd_add_meta_context before "
                  "connecting or the server does not support it");
