@@ -1,6 +1,6 @@
 (* hey emacs, this is OCaml code: -*- tuareg -*- *)
 (* libnbd OCaml test case
- * Copyright (C) 2013-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -62,22 +62,28 @@ let () =
   let meta = NBD.can_meta_context nbd NBD.context_base_allocation in
   assert meta;
 
-  (* info on something not present fails, wipes out prior info *)
-  NBD.set_export_name nbd "a";
-  fail_unary NBD.opt_info nbd;
+  (* changing export wipes out prior info *)
+  NBD.set_export_name nbd "b";
   fail_unary NBD.get_size nbd;
   fail_unary NBD.is_read_only nbd;
   fail_binary NBD.can_meta_context nbd NBD.context_base_allocation;
 
-  (* info for a different export *)
+  (* info on something not present fails *)
+  NBD.set_export_name nbd "a";
+  fail_unary NBD.opt_info nbd;
+
+  (* info for a different export, with automatic meta_context disabled *)
   NBD.set_export_name nbd "b";
+  NBD.set_request_meta_context nbd false;
   NBD.opt_info nbd;
+  (* idempotent name change is no-op *)
+  NBD.set_export_name nbd "b";
   let size = NBD.get_size nbd in
   assert (size = 1L);
   let ro = NBD.is_read_only nbd in
   assert (not ro);
-  let meta = NBD.can_meta_context nbd NBD.context_base_allocation in
-  assert meta;
+  fail_binary NBD.can_meta_context nbd NBD.context_base_allocation;
+  NBD.set_request_meta_context nbd true;
 
   (* go on something not present *)
   NBD.set_export_name nbd "a";
@@ -103,6 +109,31 @@ let () =
   fail_unary NBD.opt_info nbd;
   let size = NBD.get_size nbd in
   assert (size = 4L);
+  let meta = NBD.can_meta_context nbd NBD.context_base_allocation in
+  assert meta;
+
+  NBD.shutdown nbd;
+
+  (* Another connection. This time, check that SET_META triggered by opt_info
+   * persists through nbd_opt_go with set_request_meta_context disabled.
+   *)
+  let nbd = NBD.create () in
+  NBD.set_opt_mode nbd true;
+  NBD.connect_command nbd
+                      ["nbdkit"; "-s"; "--exit-with-parent"; "-v";
+                       "sh"; script];
+  NBD.add_meta_context nbd "x-unexpected:bogus";
+
+  fail_binary NBD.can_meta_context nbd NBD.context_base_allocation;
+  NBD.opt_info nbd;
+  let meta = NBD.can_meta_context nbd NBD.context_base_allocation in
+  assert (not meta);
+  NBD.set_request_meta_context nbd false;
+  (* Adding to the request list now won't matter *)
+  NBD.add_meta_context nbd NBD.context_base_allocation;
+  NBD.opt_go nbd;
+  let meta = NBD.can_meta_context nbd NBD.context_base_allocation in
+  assert (not meta);
 
   NBD.shutdown nbd
 
